@@ -21,10 +21,10 @@ import (
 	"strings"
 	"sync"
 
-	"github.com/gohugoio/hugo/resource"
-
 	"github.com/gohugoio/hugo/deps"
 	"github.com/gohugoio/hugo/helpers"
+	"github.com/gohugoio/hugo/langs"
+	"github.com/gohugoio/hugo/publisher"
 
 	"github.com/gohugoio/hugo/i18n"
 	"github.com/gohugoio/hugo/tpl"
@@ -130,7 +130,7 @@ func newHugoSites(cfg deps.DepsCfg, sites ...*Site) (*HugoSites, error) {
 		s.owner = h
 	}
 
-	if err := applyDepsIfNeeded(cfg, sites...); err != nil {
+	if err := applyDeps(cfg, sites...); err != nil {
 		return nil, err
 	}
 
@@ -162,7 +162,7 @@ func (h *HugoSites) initGitInfo() error {
 	return nil
 }
 
-func applyDepsIfNeeded(cfg deps.DepsCfg, sites ...*Site) error {
+func applyDeps(cfg deps.DepsCfg, sites ...*Site) error {
 	if cfg.TemplateProvider == nil {
 		cfg.TemplateProvider = tplimpl.DefaultTemplateProvider
 	}
@@ -181,8 +181,11 @@ func applyDepsIfNeeded(cfg deps.DepsCfg, sites ...*Site) error {
 			continue
 		}
 
+		cfg.Language = s.Language
+		cfg.MediaTypes = s.mediaTypesConfig
+		cfg.OutputFormats = s.outputFormatsConfig
+
 		if d == nil {
-			cfg.Language = s.Language
 			cfg.WithTemplate = s.withSiteTemplates(cfg.WithTemplate)
 
 			var err error
@@ -199,7 +202,7 @@ func applyDepsIfNeeded(cfg deps.DepsCfg, sites ...*Site) error {
 			}
 
 		} else {
-			d, err = d.ForLanguage(s.Language)
+			d, err = d.ForLanguage(cfg)
 			if err != nil {
 				return err
 			}
@@ -207,11 +210,22 @@ func applyDepsIfNeeded(cfg deps.DepsCfg, sites ...*Site) error {
 			s.Deps = d
 		}
 
-		s.resourceSpec, err = resource.NewSpec(s.Deps.PathSpec, s.mediaTypesConfig)
-		if err != nil {
+		// Set up the main publishing chain.
+		s.publisher = publisher.NewDestinationPublisher(d.PathSpec.BaseFs.PublishFs, s.outputFormatsConfig, s.mediaTypesConfig, cfg.Cfg.GetBool("minify"))
+
+		if err := s.initializeSiteInfo(); err != nil {
 			return err
 		}
 
+		siteConfig, err := loadSiteConfig(s.Language)
+		if err != nil {
+			return err
+		}
+		s.siteConfig = siteConfig
+		s.siteRefLinker, err = newSiteRefLinker(s.Language, s)
+		if err != nil {
+			return err
+		}
 	}
 
 	return nil
@@ -228,10 +242,7 @@ func NewHugoSites(cfg deps.DepsCfg) (*HugoSites, error) {
 
 func (s *Site) withSiteTemplates(withTemplates ...func(templ tpl.TemplateHandler) error) func(templ tpl.TemplateHandler) error {
 	return func(templ tpl.TemplateHandler) error {
-		templ.LoadTemplates(s.PathSpec.GetLayoutDirPath(), "")
-		if s.PathSpec.ThemeSet() {
-			templ.LoadTemplates(s.PathSpec.GetThemeDir()+"/layouts", "theme")
-		}
+		templ.LoadTemplates("")
 
 		for _, wt := range withTemplates {
 			if wt == nil {
@@ -280,8 +291,16 @@ func (h *HugoSites) reset() {
 	}
 }
 
+// resetLogs resets the log counters etc. Used to do a new build on the same sites.
+func (h *HugoSites) resetLogs() {
+	h.Log.ResetLogCounters()
+	for _, s := range h.Sites {
+		s.Deps.DistinctErrorLog = helpers.NewDistinctLogger(h.Log.ERROR)
+	}
+}
+
 func (h *HugoSites) createSitesFromConfig() error {
-	oldLangs, _ := h.Cfg.Get("languagesSorted").(helpers.Languages)
+	oldLangs, _ := h.Cfg.Get("languagesSorted").(langs.Languages)
 
 	if err := loadLanguageSettings(h.Cfg, oldLangs); err != nil {
 		return err
@@ -307,7 +326,7 @@ func (h *HugoSites) createSitesFromConfig() error {
 		s.owner = h
 	}
 
-	if err := applyDepsIfNeeded(depsCfg, sites...); err != nil {
+	if err := applyDeps(depsCfg, sites...); err != nil {
 		return err
 	}
 
@@ -347,6 +366,11 @@ type BuildCfg struct {
 // Note that a page does not have to have a content page / file.
 // For regular builds, this will allways return true.
 func (cfg *BuildCfg) shouldRender(p *Page) bool {
+	if p.forceRender {
+		p.forceRender = false
+		return true
+	}
+
 	if len(cfg.RecentlyVisited) == 0 {
 		return true
 	}
@@ -470,7 +494,7 @@ func (h *HugoSites) createMissingPages() error {
 						origKey := key
 
 						if s.Info.preserveTaxonomyNames {
-							key = s.PathSpec.MakePathSanitized(key)
+							key = s.PathSpec.MakeSegment(key)
 						}
 						for _, p := range taxonomyPages {
 							// Some people may have /authors/MaxMustermann etc. as paths.
@@ -695,7 +719,7 @@ func (m *contentChangeMap) resolveAndRemove(filename string) (string, string, bu
 	defer m.mu.RUnlock()
 
 	// Bundles share resources, so we need to start from the virtual root.
-	relPath, _ := m.pathSpec.RelContentDir(filename)
+	relPath := m.pathSpec.RelContentDir(filename)
 	dir, name := filepath.Split(relPath)
 	if !strings.HasSuffix(dir, helpers.FilePathSeparator) {
 		dir += helpers.FilePathSeparator
@@ -721,6 +745,11 @@ func (m *contentChangeMap) resolveAndRemove(filename string) (string, string, bu
 		}
 	}
 
+	if isContent && fileTp != bundleNot {
+		// A new bundle.
+		return dir, dir, fileTp
+	}
+
 	// Not part of any bundle
 	return dir, filename, bundleNot
 }
